﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

using HeuristicLab.Services.Authentication.DataAccess;
using HeuristicLab.Services.Authentication.DataTransfer;
using HeuristicLab.Services.Authentication;
using DA = HeuristicLab.Services.Authentication.DataAccess;
using DT = HeuristicLab.Services.Authentication.DataTransfer;
using HLSA = HeuristicLab.Services.Authentication;

namespace HeuristicLab.Services.Authentication.AdminMethods {
  public class GroupAdmin : ResourceAdmin {

    public override HashSet<Hashtable> Get(Hashtable values) {
      values[eResource.ResourceType] = "ResourceGroup";
      String QueryString = "SELECT * FROM Resource";
      List<String> lcv = new List<String>();
      String QueryStringExtension = " WHERE ";
      CreateQueryStringExtension(lcv, ref QueryStringExtension, values);
      if (QueryStringExtension.CompareTo(" WHERE ") > 0) {
        QueryString += QueryStringExtension;
      } // else select all
      object[] compValues = lcv.ToArray();
      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {
        if (cmdc.DatabaseExists()) {
          Table<DA.Resource> clTable = cmdc.GetTable<DA.Resource>();
          IEnumerable<DA.Resource> groups = cmdc.ExecuteQuery<DA.Resource>(QueryString, compValues);
          HashSet<Hashtable> resultSet = new HashSet<Hashtable>();
          var selectedGroups = from c in groups orderby c.Name select HLSA.Convert.ToDto((DA.ResourceGroup)c);
          foreach (DT.ResourceGroup cl in selectedGroups) {
            Hashtable ht = new Hashtable();
            ht.Add(eResource.ResourceID, cl.ResourceID);
            ht.Add(eResource.Name, cl.Name);
            ht.Add(eResource.Description, cl.Description);
            ht.Add(eResource.ResourceType, cl.ResourceType);
            // look for group entries in ResourceResourcGroup  
            //Guid groupID = new Guid();  /// ??
            String groupID = "";
            String groupName = "";
            GetGroupIDandName(ref groupID, ref groupName, cmdc, cl.ResourceID);

            ht.Add(eResource.ResourceIDGroup, groupID);
            ht.Add(eResource.ResourceNameGroup, groupName);

            resultSet.Add(ht);
          }
          return resultSet;
        }
        return null;
      }
    }

    private void TransferData(DT.ResourceGroup c, Hashtable values) {
      c.ResourceID = new Guid((string)values[eResource.ResourceID]);
      c.Name = (string)values[eResource.Name];
      c.Description = (string)values[eResource.Description];
      c.ResourceType = "ResourceGroup";
    }

    public override bool Create(Hashtable values) {
      if ((values[eResource.ResourceID] != null) && ((string)values[eResource.ResourceID] != "")) {
        return false;
      }
      DT.ResourceGroup g = new DT.ResourceGroup();
      values[eResource.ResourceID] = Guid.NewGuid().ToString();
      TransferData(g, values);

      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {
        if (cmdc.DatabaseExists()) {
          DA.ResourceGroup gEntity = HLSA.Convert.ToEntity(g);
          cmdc.GetTable<DA.Resource>().InsertOnSubmit(gEntity);
          cmdc.SubmitChanges();

          //ResourceResourceGroup 
          if ((values[eResource.ResourceIDGroup] != null) && ((string)values[eResource.ResourceIDGroup] != "")) {
            Guid groupID = new Guid((string)values[eResource.ResourceIDGroup]);

            Table<DA.Resource> clTable = cmdc.GetTable<DA.Resource>();
            Table<ResourceResourceGroup> rrgTable = cmdc.GetTable<ResourceResourceGroup>();
            DA.Resource groupEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == groupID);
            if (groupEntity == null)
              return false;

            cmdc.SubmitChanges();

            DA.Resource clientEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == g.ResourceID);
            if (clientEntity == null)
              return false;

            //ResourceResourceGroup 
            ResourceResourceGroup rrgEntity = new ResourceResourceGroup();
            rrgEntity.ResourceGroupID = groupEntity.ResourceID;
            rrgEntity.ResourceID = clientEntity.ResourceID;
            cmdc.GetTable<ResourceResourceGroup>().InsertOnSubmit(rrgEntity);

            cmdc.SubmitChanges();
          }
          return true;
        }
        return false;
      }
    }

    public override bool Update(Hashtable values) {
      string groupIDString = (string)values[eResource.ResourceIDGroup];
      string IDString = (string)values[eResource.ResourceID];

      DT.ResourceGroup uc = new DT.ResourceGroup();
      bool groupModified = false;
      bool success = true;
      bool validGroupID = IsIDString(groupIDString);
      bool validID = (IDString != "") && (IsIDString(IDString));

      if (!validGroupID || !validID || (IDString.CompareTo(groupIDString) == 0))
        return false;

      Guid resID = new Guid(IDString);
      TransferData(uc, values);

      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {

        if (cmdc.DatabaseExists()) {
          Guid currentGroupID = new Guid(groupIDString);
          Guid oldGroupID = new Guid();

          ResourceResourceGroup rrgEntity = cmdc.GetTable<ResourceResourceGroup>().FirstOrDefault(
                               x => x.ResourceID == resID);
          
          if (rrgEntity != null) {
            oldGroupID = rrgEntity.ResourceGroupID;
          }
          // check if groupname was modified
          groupModified = (oldGroupID != currentGroupID);
          DA.Resource resEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == uc.ResourceID);
          // update client
          if ((resEntity != null) && success) {
            uc.ResourceType = resEntity.ResourceType; // dont allow change of ResourceTyp
            HLSA.Convert.ToEntity(uc, (DA.ResourceGroup)resEntity);
            if (groupModified) {
              // remove entry in ResourceResourcGroup if group modiied
              RemoveRRGEntry(cmdc, resID);              
            }
            cmdc.SubmitChanges();
          }

          if (groupModified && success) {
            CreateRRGEntry(cmdc, resID, currentGroupID);
            cmdc.SubmitChanges();
          }
        }
      }
      return success;
    }
  }
}
